]> sigrok.org Git - libsigrokdecode.git/blobdiff - decode.c
Bugfix: Add missing Py_INCREF()s.
[libsigrokdecode.git] / decode.c
index 536be2c347315c4ac941d0490d5c307b17ab8733..5289470670a227606666bdf0a7353769fd4bdab5 100644 (file)
--- a/decode.c
+++ b/decode.c
@@ -22,6 +22,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <dirent.h>
+#include <config.h>
 
 /* Re-define some string functions for Python >= 3.0. */
 #if PY_VERSION_HEX >= 0x03000000
 #endif
 
 /* The list of protocol decoders. */
-GSList *list_pds;
+GSList *list_pds = NULL;
+
+/*
+ * Here's a quick overview of Python/C API reference counting.
+ *
+ * Check the Python/C API docs for what type of reference a function returns.
+ *
+ *  - If it returns a "new reference", you're responsible to Py_DECREF() it.
+ *
+ *  - If it returns a "borrowed reference", you MUST NOT Py_DECREF() it.
+ *
+ *  - If a function "steals" a reference, you no longer are responsible for
+ *    Py_DECREF()ing it (someone else will do it for you at some point).
+ */
 
 /**
  * Initialize libsigrokdecode.
@@ -47,22 +61,20 @@ int sigrokdecode_init(void)
        /* Py_Initialize() returns void and usually cannot fail. */
        Py_Initialize();
 
-       /* Add some more search directories for convenience. */
+       /* Add search directory for the protocol decoders. */
        /* FIXME: Check error code. */
        /* FIXME: What happens if this function is called multiple times? */
-       PyRun_SimpleString(
-               "import sys;"
-               "sys.path.append('libsigrokdecode/decoders');"
-               "sys.path.append('" DECODERS_DIR "');"
-               );
+       PyRun_SimpleString("import sys;"
+                          "sys.path.append(r'" DECODERS_DIR "');");
 
        if (!(dir = opendir(DECODERS_DIR)))
                return SIGROKDECODE_ERR_DECODERS_DIR;
 
        while ((dp = readdir(dir)) != NULL) {
-               if (!strstr(dp->d_name, ".py"))
+               if (!g_str_has_suffix(dp->d_name, ".py"))
                        continue;
-               if ((tmp = strdup(dp->d_name)))
+               /* For now use the filename (without .py) as decoder name. */
+               if ((tmp = g_strndup(dp->d_name, strlen(dp->d_name) - 3)))
                        list_pds = g_slist_append(list_pds, tmp);
        }
        closedir(dir);
@@ -70,12 +82,24 @@ int sigrokdecode_init(void)
        return SIGROKDECODE_OK;
 }
 
+/**
+ * Returns the list of supported/loaded protocol decoders.
+ *
+ * This is a GSList containing the names of the decoders as strings.
+ *
+ * @return List of decoders, NULL if none are supported or loaded.
+ */
+GSList *sigrokdecode_list_decoders(void)
+{
+       return list_pds;
+}
+
 /**
  * Helper function to handle Python strings.
  *
  * TODO: @param entries.
  *
* @return LIBSIGROKDECODE_OK upon success, a (negative) error code otherwise.
  *         The 'outstr' argument points to a malloc()ed string upon success.
  */
 static int h_str(PyObject *py_res, PyObject *py_func, PyObject *py_mod,
@@ -87,15 +111,20 @@ static int h_str(PyObject *py_res, PyObject *py_func, PyObject *py_mod,
        py_str = PyMapping_GetItemString(py_res, (char *)key);
        if (!py_str || !PyString_Check(py_str)) {
                if (PyErr_Occurred())
-                       PyErr_Print();
+                       PyErr_Print(); /* Returns void. */
                Py_DECREF(py_func);
                Py_DECREF(py_mod);
                return SIGROKDECODE_ERR_PYTHON; /* TODO: More specific error? */
        }
 
+       /*
+        * PyString_AsString()'s returned string refers to an internal buffer
+        * (not a copy), i.e. the data must not be modified, and the memory
+        * must not be free()'d.
+        */
        if (!(str = PyString_AsString(py_str))) {
                if (PyErr_Occurred())
-                       PyErr_Print();
+                       PyErr_Print(); /* Returns void. */
                Py_DECREF(py_str);
                Py_DECREF(py_func);
                Py_DECREF(py_mod);
@@ -104,7 +133,7 @@ static int h_str(PyObject *py_res, PyObject *py_func, PyObject *py_mod,
 
        if (!(*outstr = strdup(str))) {
                if (PyErr_Occurred())
-                       PyErr_Print();
+                       PyErr_Print(); /* Returns void. */
                Py_DECREF(py_str);
                Py_DECREF(py_func);
                Py_DECREF(py_mod);
@@ -121,7 +150,7 @@ static int h_str(PyObject *py_res, PyObject *py_func, PyObject *py_mod,
  *
  * @param name TODO
  *
* @return LIBSIGROKDECODE_OK upon success, a (negative) error code otherwise.
  */
 int sigrokdecode_load_decoder(const char *name,
                              struct sigrokdecode_decoder **dec)
@@ -131,31 +160,31 @@ int sigrokdecode_load_decoder(const char *name,
        int r;
 
        /* Get the name of the decoder module as Python string. */
-       if (!(py_name = PyString_FromString(name))) {
-               PyErr_Print();
+       if (!(py_name = PyString_FromString(name))) { /* NEWREF */
+               PyErr_Print(); /* Returns void. */
                return SIGROKDECODE_ERR_PYTHON; /* TODO: More specific error? */
        }
 
        /* "Import" the Python module. */
-       if (!(py_mod = PyImport_Import(py_name))) {
-               PyErr_Print();
+       if (!(py_mod = PyImport_Import(py_name))) { /* NEWREF */
+               PyErr_Print(); /* Returns void. */
                Py_DECREF(py_name);
                return SIGROKDECODE_ERR_PYTHON; /* TODO: More specific error? */
        }
        Py_DECREF(py_name);
 
        /* Get the 'register' function name as Python callable object. */
-       py_func = PyObject_GetAttrString(py_mod, "register");
+       py_func = PyObject_GetAttrString(py_mod, "register"); /* NEWREF */
        if (!py_func || !PyCallable_Check(py_func)) {
                if (PyErr_Occurred())
-                       PyErr_Print();
+                       PyErr_Print(); /* Returns void. */
                Py_DECREF(py_mod);
                return SIGROKDECODE_ERR_PYTHON; /* TODO: More specific error? */
        }
 
        /* Call the 'register' function without arguments, get the result. */
-       if (!(py_res = PyObject_CallFunction(py_func, NULL))) {
-               PyErr_Print();
+       if (!(py_res = PyObject_CallFunction(py_func, NULL))) { /* NEWREF */
+               PyErr_Print(); /* Returns void. */
                Py_DECREF(py_func);
                Py_DECREF(py_mod);
                return SIGROKDECODE_ERR_PYTHON; /* TODO: More specific error? */
@@ -179,10 +208,10 @@ int sigrokdecode_load_decoder(const char *name,
        Py_DECREF(py_func);
 
        /* Get the 'decode' function name as Python callable object. */
-       py_func = PyObject_GetAttrString(py_mod, "decode");
+       py_func = PyObject_GetAttrString(py_mod, "decode"); /* NEWREF */
        if (!py_func || !PyCallable_Check(py_func)) {
                if (PyErr_Occurred())
-                       PyErr_Print();
+                       PyErr_Print(); /* Returns void. */
                Py_DECREF(py_mod);
                return SIGROKDECODE_ERR_PYTHON; /* TODO: More specific error? */
        }
@@ -205,7 +234,7 @@ int sigrokdecode_load_decoder(const char *name,
  * @param outbuf TODO
  * @param outbuflen TODO
  *
* @return LIBSIGROKDECODE_OK upon success, a (negative) error code otherwise.
  */
 int sigrokdecode_run_decoder(struct sigrokdecode_decoder *dec,
                             uint8_t *inbuf, uint64_t inbuflen,
@@ -230,13 +259,13 @@ int sigrokdecode_run_decoder(struct sigrokdecode_decoder *dec,
 
        /* TODO: Error handling. */
        py_mod = dec->py_mod;
+       Py_INCREF(py_mod);
        py_func = dec->py_func;
-
-       /* TODO: Really run Py_DECREF on py_mod/py_func? */
+       Py_INCREF(py_func);
 
        /* Create a Python tuple of size 1. */
-       if (!(py_args = PyTuple_New(1))) {
-               PyErr_Print();
+       if (!(py_args = PyTuple_New(1))) { /* NEWREF */
+               PyErr_Print(); /* Returns void. */
                Py_DECREF(py_func);
                Py_DECREF(py_mod);
                return SIGROKDECODE_ERR_PYTHON; /* TODO: More specific error? */
@@ -244,26 +273,30 @@ int sigrokdecode_run_decoder(struct sigrokdecode_decoder *dec,
 
        /* Get the input buffer as Python "string" (byte array). */
        /* TODO: int vs. uint64_t for 'inbuflen'? */
-       if (!(py_value = Py_BuildValue("s#", inbuf, inbuflen))) {
-               PyErr_Print();
+       if (!(py_value = Py_BuildValue("s#", inbuf, inbuflen))) { /* NEWREF */
+               PyErr_Print(); /* Returns void. */
                Py_DECREF(py_args);
                Py_DECREF(py_func);
                Py_DECREF(py_mod);
                return SIGROKDECODE_ERR_PYTHON; /* TODO: More specific error? */
        }
 
-       if (PyTuple_SetItem(py_args, 0, py_value) != 0) {
-               PyErr_Print();
-               Py_DECREF(py_value);
+       /*
+        * IMPORTANT: PyTuple_SetItem() "steals" a reference to py_value!
+        * That means we are no longer responsible for Py_DECREF()'ing it.
+        * It will automatically be free'd when the 'py_args' tuple is free'd.
+        */
+       if (PyTuple_SetItem(py_args, 0, py_value) != 0) { /* STEAL */
+               PyErr_Print(); /* Returns void. */
+               Py_DECREF(py_value); /* TODO: Ref. stolen upon error? */
                Py_DECREF(py_args);
                Py_DECREF(py_func);
                Py_DECREF(py_mod);
                return SIGROKDECODE_ERR_PYTHON; /* TODO: More specific error? */
        }
 
-       if (!(py_res = PyObject_CallObject(py_func, py_args))) {
-               PyErr_Print();
-               Py_DECREF(py_value);
+       if (!(py_res = PyObject_CallObject(py_func, py_args))) { /* NEWREF */
+               PyErr_Print(); /* Returns void. */
                Py_DECREF(py_args);
                Py_DECREF(py_func);
                Py_DECREF(py_mod);
@@ -272,9 +305,8 @@ int sigrokdecode_run_decoder(struct sigrokdecode_decoder *dec,
 
        if ((ret = PyObject_AsCharBuffer(py_res, (const char **)outbuf,
                                         (Py_ssize_t *)outbuflen))) {
-               PyErr_Print();
+               PyErr_Print(); /* Returns void. */
                Py_DECREF(py_res);
-               Py_DECREF(py_value);
                Py_DECREF(py_args);
                Py_DECREF(py_func);
                Py_DECREF(py_mod);
@@ -282,7 +314,6 @@ int sigrokdecode_run_decoder(struct sigrokdecode_decoder *dec,
        }
 
        Py_DECREF(py_res);
-       // Py_DECREF(py_value);
        Py_DECREF(py_args);
        Py_DECREF(py_func);
        Py_DECREF(py_mod);
@@ -293,7 +324,7 @@ int sigrokdecode_run_decoder(struct sigrokdecode_decoder *dec,
 /**
  * Shutdown libsigrokdecode.
  *
* @return LIBSIGROKDECODE_OK upon success, a (negative) error code otherwise.
  */
 int sigrokdecode_shutdown(void)
 {
@@ -70,12 +82,24 @@ int sigrokdecode_init(void)
  * TODO: @param entries.
  *
- * @return LIBSIGROKDECODE_OK upon success, a (negative) error code otherwise.
+ * @return SIGROKDECODE_OK upon success, a (negative) error code otherwise.
  *         The 'outstr' argument points to a malloc()ed string upon success.
  */
@@ -121,7 +150,7 @@ static int h_str(PyObject *py_res, PyObject *py_func, PyObject *py_mod,
  * @param name TODO
  *
- * @return LIBSIGROKDECODE_OK upon success, a (negative) error code otherwise.
+ * @return SIGROKDECODE_OK upon success, a (negative) error code otherwise.
  */
 int sigrokdecode_load_decoder(const char *name,
@@ -205,7 +234,7 @@ int sigrokdecode_load_decoder(const char *name,
  * @param outbuflen TODO
  *
- * @return LIBSIGROKDECODE_OK upon success, a (negative) error code otherwise.
+ * @return SIGROKDECODE_OK upon success, a (negative) error code otherwise.
  */
 int sigrokdecode_run_decoder(struct sigrokdecode_decoder *dec,
@@ -293,7 +324,7 @@ int sigrokdecode_run_decoder(struct sigrokdecode_decoder *dec,
  * Shutdown libsigrokdecode.
  *
- * @return LIBSIGROKDECODE_OK upon success, a (negative) error code otherwise.
+ * @return SIGROKDECODE_OK upon success, a (negative) error code otherwise.
  */
 int sigrokdecode_shutdown(void)